#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.



import ROOT
ROOT.gROOT.SetStyle('Plain')

from TDetector import TEEEDetector
from __logging__  import logger



class TEventCanvas(ROOT.TCanvas):

    """
    """

    NAME   = 'DisplayCanvas'
    TITLE  = 'EEE Event Display'
    WIDTH  = 1270
    HEIGHT = 790
    
    def __init__(self):
        """ Constructor.
        """
        ROOT.TCanvas.__init__(self, self.NAME, self.TITLE,
                              self.WIDTH, self.HEIGHT)
        self.setupPads()
        self.GeoManager = ROOT.TGeoManager('Geometry manager',
                                           'Geometry manager')
        self.DetectorLayout = TEEEDetector()
        self.GeoManager.SetTopVolume(self.DetectorLayout)
        self.GeoManager.CloseGeometry()
        self.GeoManager.SetVisLevel(4)
        self.GeoManager.SetVisOption(0)
        self.DisplayObjList = []

    def setupPads(self, sideAspectRatio = 1.9):
        """ Setup the pads.
        """
        width  = 2*self.WIDTH  - self.GetWw()
        height = 2*self.HEIGHT - self.GetWh()
        self.SetWindowSize(width, height)
        self.PadList = []
        x = sideAspectRatio*(self.HEIGHT/2.)/self.WIDTH
        y = 1 - (1-x)*self.WIDTH/self.HEIGHT
        pad = ROOT.TPad('frontPad', 'Front Pad', 0.0, 0.5, x, 1.0)
        pad.Draw()
        self.PadList.append(pad)
        pad = ROOT.TPad('sidePad' , 'Side Pad' , 0.0, 0.0, x, 0.5)
        pad.Draw()
        self.PadList.append(pad)
        pad = ROOT.TPad('topPad'  , 'Top Pad'  , x, y, 1.0, 1.0)
        pad.Draw()
        self.PadList.append(pad)
        pad = ROOT.TPad('textPad' , 'Text Pad' , x, 0.0, 1.0, y)
        pad.Draw()
        self.PadList.append(pad)
        self.Update()

    def cd(self, i):
        """
        """
        self.PadList[i - 1].cd()

    def clear(self):
        """
        """
        self.DisplayObjList = []
        self.PadList[3].Clear()

    def update(self):
        """
        """
        for (i, pad) in enumerate(self.PadList):
            self.cd(i + 1)
            pad.Modified()
        ROOT.TCanvas.cd(self)
        self.Update()

    def setView(self, view = None, zoom = 1.55):
        """
        """
        pad = ROOT.gPad
        view3d = pad.GetView()
        if view is not None:
            exec('view3d.%s()' % view)
            view3d.SetParallel()
        view3d.ZoomView(0, zoom)
        #view3d.AdjustPad()
        view3d.AdjustScales()

    def displayRefArrows(self):
        """
        """
        self.HorArrowSide = ROOT.TArrow(0, -0.4, 0.075, -0.4, 0.01, '-|>')
        self.VertArrowSide = ROOT.TArrow(0, -0.4, 0, -0.25, 0.01, '-|>')
        self.HorArrowTop = ROOT.TArrow(0, 0, 0.1, 0, 0.01, '-|>')
        self.VertArrowTop = ROOT.TArrow(0, 0, 0, 0.1, 0.01, '-|>')
        self.XLabelSide = ROOT.TLatex(0.08, -0.4, 'x')
        self.XLabelSide.SetTextAlign(12)
        self.YLabelSide = ROOT.TLatex(0.08, -0.4, 'y')
        self.YLabelSide.SetTextAlign(12)
        self.ZLabelSide = ROOT.TLatex(0, -0.24, 'z')
        self.ZLabelSide.SetTextAlign(21)
        self.XLabelTop = ROOT.TLatex(0.11, 0, 'x')
        self.XLabelTop.SetTextAlign(12)
        self.XLabelTop.SetTextSize(0.75*self.XLabelTop.GetTextSize())
        self.YLabelTop = ROOT.TLatex(0, 0.115, 'y')
        self.YLabelTop.SetTextAlign(21)
        self.YLabelTop.SetTextSize(0.75*self.YLabelTop.GetTextSize())
        self.cd(1)
        self.HorArrowSide.Draw()
        self.XLabelSide.Draw()
        self.VertArrowSide.Draw()
        self.ZLabelSide.Draw()
        self.cd(2)
        self.HorArrowSide.Draw()
        self.YLabelSide.Draw()
        self.VertArrowSide.Draw()
        self.ZLabelSide.Draw()
        self.cd(3)
        self.HorArrowTop.Draw()
        self.XLabelTop.Draw()
        self.VertArrowTop.Draw()
        self.YLabelTop.Draw()
        
    def displayDetectorLayout(self):
        """
        """
        self.cd(1); self.DetectorLayout.Draw(); self.setView('Front')
        self.cd(2); self.DetectorLayout.Draw(); self.setView('Side')
        self.cd(3); self.DetectorLayout.Draw(); self.setView('Top')
        self.displayRefArrows()
        self.Update()

    def displayObject(self, obj, pads = [1, 2, 3]):
        """
        """
        self.DisplayObjList.append(obj)
        for pad in pads:
            self.cd(pad)
            obj.display()

    def displayInfo(self, runId, evtId, infoDict = {}, wrap = 10):
        self.cd(4)
        text = 'Trigger Number %s -- Event Number %s' % (runId, evtId)
        latex = ROOT.TLatex().DrawLatex(0.1, 0.9, text)
        latex.SetTextSize(1.4*latex.GetTextSize())
        self.DisplayObjList.append(latex)
        keys = infoDict.keys()
        keys.sort()
        j = -1
        for (i, key) in enumerate(keys):
            if i % wrap == 0:
                j += 1
            text = '%s: %s' % (key, infoDict[key])
            x = 0.1 + j*0.5
            y = 0.75 - 0.075*(i - j*wrap)
            latex = ROOT.TLatex().DrawLatex(x, y, text)
            self.DisplayObjList.append(latex)
        self.Update()



if __name__ == '__main__':
    canvas = TEventCanvas()
    canvas.displayDetectorLayout()
