/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3BaseEvent_H
#define E3BaseEvent_H

#include <math.h>

#include "E3Timestamp.h"
#include "E3BitMask.h"
#include "E3.h"


/// \brief Definition of the bits for the event status bit mask.
enum StatusBit {
  RUN_INFO_ASSIGNED = 8,
  EVENT_ID_UNPACKED = 9,
  TIMESTAMP_UNPACKED = 10,
  PPS_COUNT_UNPACKED = 11,
  TIMESTAMP_CORRECTED = 12,
  UNPACK_SUCCESSFUL = 13,
  FIND_HITS_SUCCESSFUL = 16,
  FIND_CLUSTERS_SUCCESSFUL = 17,
  FIND_TRACKS_SUCCESSFUL = 18
};


/// \brief Definition of the bits for the event error bit mask.
enum ErrorBit {
  UNPACK_BOARD_BEGIN_MISMATCH = 8,
  UNPACK_BOARD_END_MISMATCH   = 9,
  UNPACK_TDC_END_MISSING      = 10
};


/*! \class E3BaseEvent
  \brief Base class for a minimal event structure.

  This class encapsulates all the necessary information to unambiguously
  identify a given event (i.e., the machine id, the run id and the event id).
  In addition to that, it also stores the event timestamp, which is the
  other piece of information needed in order to be able to search for
  coincidences between different streams of data (the idea is that we can
  write to a file a list of E3BaseEvent objects and use that file for
  searching for coincidences).
*/

class E3BaseEvent
{
 public:
  /// \brief Default constructor.
  E3BaseEvent();

  /// \brief Constructor passing all the parameters.
  E3BaseEvent(E3::Station_t stationId, E3::Run_t runId,
              E3::Event_t eventId, E3Timestamp timestamp);

  /// \brief Destructor.
  ~E3BaseEvent() {}

  /// \brief Return the station id.
  inline E3::Station_t stationId() const
  { return m_stationId; }

  /// \brief Return the run id.
  inline E3::Run_t runId() const
  { return m_runId; }

  /// \brief Return the event id.
  inline E3::Event_t eventId() const
  { return m_eventId; }
  
  /// \brief Return the raw timestamp.
  inline E3Timestamp rawTimestamp() const
  { return m_rawTimestamp; }

  /// \brief Return the timestamp corrected for the GPS drift.
  inline E3Timestamp timestamp() const
  { return m_timestamp; }

  /// \brief Return the second portion of the timestamp.
  inline long long seconds() const
  { return m_timestamp.seconds(); }
  
  /// \brief Return the nanosecond portion of the timestamp.
  inline long nanoseconds() const
  { return m_timestamp.nanoseconds(); }

  /// \brief Return the status mask;
  inline E3BitMask statusMask() const
  { return m_statusMask; }

  /// \brief Return the error mask.
  inline E3BitMask errorMask() const
  { return m_errorMask; }

  /// \brief Set the station id.
  inline void setStationId(E3::Station_t stationId)
  { m_stationId = stationId; }

  /// \brief Set the run id.
  inline void setRunId(E3::Run_t runId)
  { m_runId = runId; }

  /// \brief Set the event id.
  inline void setEventId(E3::Event_t eventId)
  { m_eventId = eventId; }

  /// \brief Set the timestamp.
  inline void setRawTimestamp(E3Timestamp timestamp)
  { m_rawTimestamp = timestamp; }

  /// \brief Set the timestamp from seconds and nanoseconds.
  inline void setRawTimestamp(long long seconds, long nanoseconds)
  { m_rawTimestamp = E3Timestamp(seconds, nanoseconds); }

  /// \brief Set the timestamp.
  inline void setTimestamp(E3Timestamp timestamp)
  { m_timestamp = timestamp; }

  /// \brief Set the timestamp from seconds and nanoseconds.
  inline void setTimestamp(long long seconds, long nanoseconds)
  { m_timestamp = E3Timestamp(seconds, nanoseconds); }

  /// \brief Add an offset to the corrected timestamp.
  inline void offsetTimestamp(E3Timestamp offset)
  { m_timestamp += offset; }

  /// \brief Add an offset to the corrected timestamp.
  inline void offsetTimestamp(long long seconds, long nanoseconds = 0)
  { m_timestamp += E3Timestamp(seconds, nanoseconds); }

  /// \brief Check a bit in the event status bit mask.
  inline bool checkStatusBit(unsigned int position) const
  { return m_statusMask.checkBit(position); } 

  /// \brief Set a bit in the event status bit mask.
  inline void setStatusBit(unsigned int position)
  { return m_statusMask.setBit(position); }

  /// \brief Check a bit in the event error bit mask.
  inline bool checkErrorBit(unsigned int position) const
  { return m_errorMask.checkBit(position); }

  /// \brief Set a bit in the event error bit mask.
  inline void setErrorBit(unsigned int position)
  { return m_errorMask.setBit(position); }

  /// \brief Return true if the event has errors.
  inline bool hasErrors() const { return m_errorMask.payload() > 0; }

  /// \brief Comparison operators.
  ///
  /// \todo We should consider whether we want == to be a *real* eguality
  /// (i.e., enforce the equality of all the class members, including
  /// runId and eventId). If this is confusing we should probably get rid
  /// of the operator altogether.
  friend bool operator==(const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return (lhs.timestamp() == lhs.timestamp()); }

  friend bool operator!=(const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return !operator==(lhs,rhs);} 

  /// \brief The inequality operators, in all the flavors, are based
  /// on the timestamps only.
  friend bool operator< (const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return (lhs.timestamp() < rhs.timestamp()); }

  friend bool operator> (const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return operator<(rhs,lhs); }

  friend bool operator<=(const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return !operator>(lhs,rhs); }

  friend bool operator>=(const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return !operator<(lhs,rhs); }

  /// \brief Time difference between two events.
  friend double delta(const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return double(lhs.timestamp() - rhs.timestamp()); }

  friend double distance(const E3BaseEvent &lhs, const E3BaseEvent &rhs)
  { return fabs(delta(lhs, rhs)); }

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3BaseEvent& e)
  { return e.fillStream(os); }

 protected:
  /// \brief The station id.
  E3::Station_t m_stationId;

  /// \brief The run id.
  E3::Run_t m_runId;

  /// \brif The event id.
  E3::Event_t m_eventId;

  /// \brief The raw timestamp from the s/ns counters.
  E3Timestamp m_rawTimestamp;

  /// \brief The timestamp corrected for the clock drift between consecutive
  /// PPS signals.
  E3Timestamp m_timestamp;

  /// \brief The bitmask for the event status.
  E3BitMask m_statusMask;

  /// \brief The bitmask for the event errors.
  E3BitMask m_errorMask;
};

#endif
