/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3DigiEvent.h"


E3DigiEvent::E3DigiEvent()
{
  clear();
}


E3DigiEvent::E3DigiEvent(E3::UnixTime_t unixTime)
{
  clear();
  m_unixTime = unixTime;
}


void E3DigiEvent::clear()
{
  m_unixTime = 0;
  m_digiWordVec.clear();
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    m_stripDataVec[plane].clear();
  }
}


void E3DigiEvent::unpackingError(E3::Board_t board,
                                 const E3DigiWordVecIter &digi,
                                 const std::ostringstream &msg)
{
  LOG(ERROR) << "Unpacking error for event " << m_stationId << "-"
             << m_runId << "-" << m_eventId << ", board " << board
             << " at word " << digi - m_digiWordVec.begin() << "/"
             << m_digiWordVec.end() - m_digiWordVec.begin() - 1
             << " (" << *digi << "): " << msg.str()
             << ". Full event dump following...\n" << *this << "\n";
}


StatusCode E3DigiEvent::unpack(const E3RunInfo& runInfo)
{
  m_stationId = runInfo.stationId();
  m_runId = runInfo.runId();
  setStatusBit(StatusBit::RUN_INFO_ASSIGNED);
  // We need to organize the data by plane and channel so that we 
  // can group together the information for creating the E3StripData object
  // to fill the m_stripDataVec class member with.
  E3DigiMap digiMap;
  // Loop over the digi words.
  E3DigiWordVecIter digi = m_digiWordVec.begin();
  /// \todo The number of boards should be defined in E3.h.
  for (E3::Board_t board = 0; board < 2; board++) {
    // Skip three (apparently useless) words.
    digi += 3;
    // Here we expect to find the board global begin.
    if (!digi->isBoardBegin()) {
      setErrorBit(ErrorBit::UNPACK_BOARD_BEGIN_MISMATCH);
      std::ostringstream msg;
      msg << "board begin (" << E3DigiWord(DigiWordType::BOARD_BEGIN)
          << ") expected instead";
      unpackingError(board, digi, msg);
      return StatusCode::FAILURE;
    }
    digi++;
    // Horrible hack to take into account the clock frequency. The issue is
    // that we have to convert a number of clock cycles to ns and we don't have
    // the information about the clock frequency. We try and guess by looking
    // ahead four positions for the pps counter and rounding the value to the
    // closest integer frequency in MHz, but at some point the number should be
    // properly passed along with the run info.
    unsigned short clockFrequency = ((digi + 4)->payload() + 500000)/1000000;
    double clockPeriod = 1000./clockFrequency;
    // End of hack.
    m_eventId = (digi++)->payload();
    m_triggerId = (digi++)->payload();
    setStatusBit(StatusBit::EVENT_ID_UNPACKED);  
    m_secondCount = (digi++)->payload();
    m_nanosecondCount = (digi++)->ns(clockPeriod);
    E3Timestamp rawTimestamp(m_secondCount, m_nanosecondCount);
    rawTimestamp += runInfo.startRunGpsRecord().timestamp();
    setRawTimestamp(rawTimestamp);
    setStatusBit(StatusBit::TIMESTAMP_UNPACKED);
    m_ppsCount = (digi++)->ppsns(clockPeriod);
    setStatusBit(StatusBit::PPS_COUNT_UNPACKED);
    // Correct for the drift of the GPS since the last PPS.
    double drift = (1000000000. - m_ppsCount)*m_nanosecondCount*1e-9 + 0.5;
    setTimestamp(rawTimestamp + E3Timestamp(0, (long)drift));
    setStatusBit(StatusBit::TIMESTAMP_CORRECTED);
    m_timeSinceStartRun = double(m_timestamp -
                                 runInfo.startRunGpsRecord().timestamp());
    // Now loop over the (up to) four TDCs in the board.
    while (!(digi++)->isTdcBegin()) {
      if (digi >= m_digiWordVec.end()) {
        std::ostringstream msg;
        msg << "end reached while expecting TDC begin";
        unpackingError(board, digi, msg);
        return StatusCode::FAILURE;
      }
      if ((digi)->isBoardEnd()) break;
      E3::Tdc_t tdc = (digi++)->tdc();
      if (digi >= m_digiWordVec.end()) {
          std::ostringstream msg;
          msg << "end reached while expecting TDC end";
          unpackingError(board, digi, msg);
          return StatusCode::FAILURE;
      }
      E3::Plane_t plane = runInfo.ninoMap().plane(board, tdc);
      bool side = runInfo.ninoMap().side(board, tdc);
      while (!(digi)->isTdcEnd(tdc)) {
        if (digi >= m_digiWordVec.end()) {
          std::ostringstream msg;
          msg << "end reached while expecting TDC end";
          unpackingError(board, digi, msg);
          return StatusCode::FAILURE;
        }
        if (digi->isBoardBegin()) {
          setErrorBit(ErrorBit::UNPACK_TDC_END_MISSING);
          std::ostringstream msg;
          msg << "board begin marker found with no prior TDC end "
              << "for TDC " << tdc;
          unpackingError(board, digi, msg);
          return StatusCode::FAILURE;
        } else if (digi->isBoardEnd()) { 
          std::ostringstream msg;
          msg << "board end marker found with no prior TDC end "
              << "for TDC " << tdc;
          unpackingError(board, digi, msg);
          return StatusCode::FAILURE;
        } else if (digi->isEdge()) {
	  E3DigiMapKey key = std::make_pair(plane, digi->channel());
	  digiMap[key].push_back(std::make_pair(side, *digi));
        } 
        digi ++;
        if (digi >= m_digiWordVec.end()) {
          std::ostringstream msg;
          msg << "no details (1)";
          unpackingError(board, digi, msg);
          return StatusCode::FAILURE;
        }
      }
    }
    // There are two more (redundant) board ends, here.
    digi++;
    if (digi >= m_digiWordVec.end()) {
      std::ostringstream msg;
      msg << "end reached while expecting two more board end markers";
      unpackingError(board, digi, msg);
      return StatusCode::FAILURE;
    }
    if (!(digi++)->isBoardEnd()) {
      std::ostringstream msg;
      msg << "no details (2)";
      unpackingError(board, digi, msg);
      return StatusCode::FAILURE;
    }
    if (digi >= m_digiWordVec.end()) {
      std::ostringstream msg;
      msg << "end reached while expecting one more board end marker";
      unpackingError(board, digi, msg);
      return StatusCode::FAILURE;
    }
    if (!(digi++)->isBoardEnd()) {
      std::ostringstream msg;
      msg << "no details (3)";
      unpackingError(board, digi, msg);
      return StatusCode::FAILURE;
    }
    // If only one board (instead of two) is connected we should be at the
    // end of the datagram and we have to break the loop over the boards.
    if (digi == m_digiWordVec.end()) break;
  }
  // And if everything went fine we must be at the end of the datagram, here.
  if (digi != m_digiWordVec.end()) {
    std::ostringstream msg;
    msg << "no details (4)";
    unpackingError(-1, digi, msg);
    return StatusCode::FAILURE;
  }

  // Finally: we're ready to parse the digiMap object and fill the
  // m_stripDataVec class member.
  for (E3DigiMapIter item = digiMap.begin(); item != digiMap.end(); item++) {
    E3StripData stripData(item->first.first, item->first.second);
    E3DigiMapValue mapValue = item->second;
    for (E3DigiMapValueIter value = mapValue.begin(); value != mapValue.end();
	 value++) {
      stripData.addEdgeDigi(value->first, value->second);
    }
    addStripData(item->first.first, stripData);
  }
  setStatusBit(StatusBit::UNPACK_SUCCESSFUL);
  return StatusCode::SUCCESS;
}


void E3DigiEvent::addDigiWord(E3::DigiWord_t digi)
{
  m_digiWordVec.push_back(E3DigiWord(digi));
}


void E3DigiEvent::addDigiWord(E3DigiWord digi)
{
  m_digiWordVec.push_back(digi);
}


void E3DigiEvent::addStripData(E3::Plane_t plane, E3StripData data)
{
  m_stripDataVec[plane].push_back(data);
}


unsigned short E3DigiEvent::numStripData(E3::Plane_t plane) const
{
  return m_stripDataVec[plane].size();
}


unsigned short E3DigiEvent::numStripData() const
{
  unsigned short n = 0;
  for (E3::Plane_t plane = 0; plane < 3; plane++) {
    n += numStripData(plane);
  }
  return n;
}


unsigned int E3DigiEvent::size() const
{
  return sizeof(E3::UnixTime_t) +
    (2 + m_digiWordVec.size())*sizeof(E3::DigiWord_t);
}


std::ostream& E3DigiEvent::fillStream(std::ostream& os) const
{
  os << "================================ "
     << "Event summary "
     << " ================================\n"
     << "Event size = " << size() << " bytes\n"
     << "Station ID = " << m_stationId << ", run ID = " << m_runId
     << ", event number = " << m_eventId << ", trigger number = "
     << m_triggerId << "\n"
     << "UNIX time = " << m_unixTime << " s, s counter = " << m_secondCount
     << " , ns counter = " << m_nanosecondCount << "\n"
     << "Last PPS arrival time = " << m_ppsCount << " ns\n"
     << "Raw timestamp       = " << m_rawTimestamp << "\n"
     << "Corrected timestamp = " << m_timestamp << "\n"
     << m_statusMask << "  [status bit mask]\n"
     << m_errorMask << "  [error bit mask]\n";
  os << "============================= "
     << "Raw binary data words"
     << " ============================\n"
     << std::hex << std::showbase;
  E3DigiWordVecIter digi, digiBegin, digiEnd;
  unsigned int position;
  digiBegin = m_digiWordVec.begin();
  digiEnd = m_digiWordVec.end();
  for (digi = digiBegin; digi != digiEnd; ++digi) {
    position = digi - digiBegin;
    os << std::setw(11) << digi->payload();
    if (position % 6 == 5 && digi != digiEnd - 1) {
      os << "  [" << std::dec << position - 5 << "-" << position << "]"
         << std::hex << "\n";
    }
  }
  os << std::setw(11*(5 - position % 6)) << "" << "  [" << std::dec
     << position - position % 6 << "-" << position << "]";
  if (numStripData() == 0) return os;
  os << std::dec << std::noshowbase << "\n"
     << "================================== "
     << "Strip data"
     << " ==================================\n";
  E3StripDataVecIter strip, stripBegin, stripEnd;
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    stripBegin = m_stripDataVec[plane].begin();
    stripEnd = m_stripDataVec[plane].end();
    for (strip = stripBegin; strip != stripEnd; strip++) {
      os << "--" << *strip;
      if (!(strip == stripEnd - 1 && plane == E3::kBotPlane)) os << "\n";
    }
  }
  return os;
}

