/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3EventList_H
#define E3EventList_H

#include <vector>
#include <algorithm>

#include "E3MessageLogger.h"
#include "E3Chrono.h"
#include "E3BaseEvent.h"
#include "E3RandomGaussGenerator.h"
#include "E3RandomTimeGenerator.h"
#include "E3CoincidenceList.h"


/*! \class E3EventList
  \brief Class describing a list of event.

  The class provides all the facilities for indexing, sorting, bisecting
  and alike.

  \todo Apparently inheriting from stl classes is a big no-no, as they do no
  provide virtual destructors. We should investigate that is some more
  details and possibly use composition instead of inheritance (also look at
  free functions). 
*/

class E3EventList : public std::vector<E3BaseEvent>
{
 public:
  /// \brief Default constructor.
  E3EventList();

  /// \brief Destructor.
  ~E3EventList() {}

  /// \brief Return the number of events in the list.
  unsigned numEvents() const { return size(); }
  
  /// \brief Add an event to the list.
  void fill(E3BaseEvent event);

  /// \brief Add an entire event list to the current event list.
  void fill(E3EventList list);

  /// \brief Insert an event in the list preserving the overall sorting.
  ///
  /// This is assumed the event list is sorted to start with.
  void insert(const E3BaseEvent &event);

  /// \brief Loop over the event list and set the run id.
  void setRunId(E3::Run_t runId);

  /// \brief Loop over the event list and set the station id.
  void setStationId(E3::Station_t stationId);

  /// \brief Re-calculate the event ids for the entire list so that
  /// they are ordered (starting from 0).
  ///
  /// This is something we probably don't want to do on real data but
  /// it might be handy when messing around (e.g. merging) Monte Carlo-generated
  /// event lists.
  void align();
  
  /// \brief Alternate align() method overwriting the station id and run id
  /// fields.
  void align(E3::Station_t stationId, E3::Run_t runId);

  /// \brief Fill the list with random (Poisson) events.
  void fillRandom(double rate, double tstart, double tstop, int seed,
                  E3::Station_t stationId = -1, E3::Run_t runId = 0);

  /// \brief Return a new event list where the timestamp are smeared
  /// with a gaussian smearing function with adjustable sigma.
  E3EventList smear(double sigma, int seed) const;

  /// \brief Sort the complete list in place.
  void sort();

  /// \brief Bisect method for binary search.
  unsigned bisect(const E3BaseEvent &event) const;

  /// \brief Search for coincidences within an event list.
  E3CoincidenceList searchCoincidences(double window);

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os, unsigned n = 3) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3EventList& l)
  { return l.fillStream(os); }

  /// \brief Search for coincidences in two event lists.
  friend E3CoincidenceList match(const E3EventList &list1,
				 const E3EventList &list2, double window);

 private:
};

#endif
