/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3MessageLogger_H
#define E3MessageLogger_H

#include <sstream>
#include <stdio.h>

enum LoggingLevel {
  DEBUG,
  INFO,
  WARNING,
  ERROR
};

/*! \class E3MessageLogger
  \brief Logging utility.
  
  This is largely mutuated by:
  http://www.drdobbs.com/cpp/logging-in-c/201804215?pgno=1

  The main idea is that we actually flush the stuff on the terminal in the
  destructor. From the page mentioned above: Why flush during destruction?
  The last two steps are important because they confer threadsafety to the
  logging mechanism. The fprintf() function is threadsafe, so even if this log
  is used from different threads, the output lines won't be scrambled. 
*/


/// \brief Set the logging level.
#define LOGGING_LEVEL DEBUG

/// \brief Macro trick to avoid accumulating stuff in the internal buffer
/// when not necessary.
#define LOG(level)                \
  if (level < LOGGING_LEVEL) ;    \
  else E3MessageLogger().get(level)


class E3MessageLogger
{
 public:
  /// \brief Default constructor.
  E3MessageLogger();
  
  /// \brief Destructor.
  virtual ~E3MessageLogger();

  /// \brief Access the internal logging buffer.
  std::ostringstream& get(LoggingLevel level = INFO);

 protected:
  /// \brief Internal std::ostringstream to accumulate logged data.
  std::ostringstream m_outputStream;

 private:
  E3MessageLogger(const E3MessageLogger&);
  E3MessageLogger& operator =(const E3MessageLogger&);

 private:
  /// \brief The level of the current message.
  LoggingLevel m_currentMsgLevel;
};

#endif

