/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3TelescopeConfiguration_H
#define E3TelescopeConfiguration_H


#include <string>
#include <iostream>

#include "E3.h"
#include "E3StatusCode.h"
#include "E3MessageLogger.h"
#include "E3TerminalFormatting.h"
#include "E3TextInputFile.h"


class E3TelescopeConfiguration
{
 public:
  /// \brief Default constructor (all the variables get assigned the default
  /// values).
  E3TelescopeConfiguration();

  /// \brief Destructor.
  ~E3TelescopeConfiguration() {}

  /// \brief Set the default values for all the data fields.
  void setDefaults();

  /// \brief Read the configuration from file.
  StatusCode read(const char* filePath);

  /// \brief Read the configuration from file.
  StatusCode read(const std::string filePath);

  /// \brief Return the z coordinate of a given plane.
  inline double z(E3::Plane_t plane) const
  { return m_z[plane]; }

  /// \brief Return the orientation angle.
  inline double orientation() const
  { return m_orientation; }

  /// \brief Retrurn the cable delay for a given plane end.
  inline int cableDelay(E3::Plane_t plane, E3::End_t end) const
  { return m_cableDelay[plane][end]; }

  /// \brief Return the number of strips per plane.
  inline int numStripsPerPlane() const
  { return m_numStripsPerPlane; }

  /// \brief Return the strip pitch.
  inline double stripPitch() const
  { return m_stripPitch; }

  /// \brief Return the strip length.
  inline double stripLength() const
  { return m_stripLength; }

  /// \brief Return the signal propagation speed along the strip.
  inline double propSpeed() const
  { return m_propSpeed; }

  /// \brief 
  inline double yoff() const
  { return 0.5*(numStripsPerPlane() - 1); }

  /// \brief 
  inline double ymax() const
  { return 0.5*m_stripLength; }

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os,
				  const E3TelescopeConfiguration& c)
  { return c.fillStream(os); }

 private:

  /// \brief The z-coordinate of the telecope planes [cm].
  double m_z[E3::kNumPlanes];

  /// \brief
  double m_layerThickness;

  /// \brief The orientation angle.
  double m_orientation;

  /// \brief The time delay [ps] compensating for the cable length.
  int m_cableDelay[E3::kNumPlanes][E3::kNumEnds];

  /// \brief Number of stripts per plane.
  int m_numStripsPerPlane;

  /// \brief The strip pitch [cm].
  double m_stripPitch;

  /// \brief The strip length [cm]
  double m_stripLength;

  /// \brief The signal propagation speed along the strips [cm/ps].
  double m_propSpeed;

};

#endif
