/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3Timestamp.h"


E3Timestamp::E3Timestamp() :
  m_seconds(0),
  m_nanoseconds(0)
{}


E3Timestamp::E3Timestamp(long long seconds, long nanoseconds) :
  m_seconds(seconds),
  m_nanoseconds(nanoseconds)
{
  // Enforce sign matching between seconds and nanoseconds.
  if (m_seconds < 0 && m_nanoseconds > 0) m_nanoseconds = - m_nanoseconds;
  // Take into account possible overflows of the ns counter.
  if (abs(m_nanoseconds) > 999999999 ) {
    m_seconds += m_nanoseconds / 1000000000;
    m_nanoseconds = m_nanoseconds % 1000000000;
  }
}

E3Timestamp::E3Timestamp(double seconds)
{
  m_seconds = floor(seconds);
  m_nanoseconds = long(1000000000*(seconds - m_seconds));
  // The floor(x) operator returns the largest integral value not greater than
  // x, i.e., no matter what, at this point m_seconds <= seconds and the
  // m_nanoseconds field is positive semidefinite. In order to enforce our
  // conventions, if m_seconds < 0 we need one more step.
  if (m_seconds < 0) {
    m_seconds += 1;
    m_nanoseconds -= 1000000000;
  }
}


E3Timestamp& E3Timestamp::operator+=(const E3Timestamp& rhs)
{
  m_seconds += rhs.seconds();
  m_nanoseconds += rhs.nanoseconds();
  // Check for overflow in the m_nanoseconds field.
  if (m_nanoseconds >= 1000000000) {
    m_nanoseconds -= 1000000000;
    m_seconds += 1;
  }
  // And we still have to enforce the sign matching between m_seconds and
  // m_nanoseconds.
  if (m_seconds < 0 && m_nanoseconds > 0) {
    m_seconds += 1;
    m_nanoseconds -= 1000000000;
  } else if (m_seconds > 0 && m_nanoseconds < 0) {
    m_seconds -= 1;
    m_nanoseconds += 1000000000;
  }
  return *this;
}


E3Timestamp& E3Timestamp::operator-=(const E3Timestamp& rhs)
{
  m_seconds -= rhs.seconds();
  m_nanoseconds -= rhs.nanoseconds();
  // Check for underflow in the m_nanoseconds field.
  if (m_nanoseconds <= -1000000000) {
    m_nanoseconds += 1000000000;
    m_seconds -= 1;
  }
  // And we still have to enforce the sign matching between m_seconds and
  // m_nanoseconds.
  if (m_seconds < 0 && m_nanoseconds > 0) {
    m_seconds += 1;
    m_nanoseconds -= 1000000000;
  } else if (m_seconds > 0 && m_nanoseconds < 0) {
    m_seconds -= 1;
    m_nanoseconds += 1000000000;
  }
  return *this;
}


std::ostream& E3Timestamp::fillStream(std::ostream& os) const
{
  os << std::showpos << m_seconds << "[s]." << std::noshowpos;
  if (m_nanoseconds != 0) {
    os << (m_nanoseconds > 0 ? "+" : "-") << std::setw(9)
       << std::setfill('0') << abs(m_nanoseconds) << std::setfill(' ');
  } else {
    os << "+000000000";
  }
  os << "[ns] (" << std::showpos << double(*this) << " s)" << std::noshowpos;
  return os;
}

