/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3Timestamp_H
#define E3Timestamp_H

#include <iostream>
#include <iomanip>
#include <cmath>
#include <stdlib.h>


/*! \class E3Timestamp
  \brief Utility class describing a timestamp.

  A timestamp is essentially a time in seconds with nanoseconds resolution.
  It has two class members, m_seconds and m_nanoseconds, and all the
  facilities for comparing, adding, subtracting, casting from and to double,
  etc. We need a dedicated class as we don't want to rely on a large double
  being consistently accurate to the nanosecond, which is what we are aiming
  to, here.
  
  Here are the basic rules of the game:
  1\ both m_seconds and m_nanoseconds are signed (as evident from the way
     they are printed out);
  2\ the cast to double is by definition m_seconds + m_nanoseconds/1000000000.;
  3\ m_nanoseconds is bound to be between -999999999 and +999999999;
  4\ when m_seconds is positive (negative), we want m_nanoseconds to be also
     positive (negative)---this is enforced in the default constructor,
     where we implicitly convert, e.g. (-1, 20000) to (-1, -20000).
  5\ when m_seconds is 0, m_nanoseconds can be either positive or negative;
     note that, for obvious reasons, (0, 20000) is *not* converted to
     (0, -20000)---i.e. you have to use the second for explicitely, if that's
     what you mean;
  6\ the previous rules determine the binary arithmetic operations (+/-),
     which preserve the ns resolution.
*/

class E3Timestamp
{
 public:
  /// \brief Default constructor.
  E3Timestamp();

  /// \brief Basic constructor.
  E3Timestamp(long long seconds, long nanoseconds);

  /// \brief Constructor from a double.
  E3Timestamp(double seconds);

  /// \brief Destructor.
  ~E3Timestamp() {}

  /// \brief Access the m_seconds field.
  inline long long seconds() const { return m_seconds; }

  /// \brief Access the m_nanoseconds field.
  inline long nanoseconds() const { return m_nanoseconds; }
 
  /// \brief Comparison operators.
  friend bool operator==(const E3Timestamp &lhs, const E3Timestamp &rhs)
  { return (lhs.seconds() == rhs.seconds() &&
	    lhs.nanoseconds() == rhs.nanoseconds()); }

  friend bool operator!=(const E3Timestamp &lhs, const E3Timestamp &rhs)
  { return !operator==(lhs,rhs); }

  friend bool operator< (const E3Timestamp &lhs, const E3Timestamp &rhs)
  { if (lhs.seconds() < rhs.seconds()) return true;
    else if (lhs.seconds() > rhs.seconds()) return false;
    else return lhs.nanoseconds() < rhs.nanoseconds();
  }

  friend bool operator> (const E3Timestamp &lhs, const E3Timestamp &rhs)
  { return operator<(rhs,lhs); }

  friend bool operator<=(const E3Timestamp &lhs, const E3Timestamp &rhs)
  { return !operator>(lhs,rhs); }

  friend bool operator>=(const E3Timestamp &lhs, const E3Timestamp &rhs)
  { return !operator<(lhs,rhs); }

  /// \brief Binary arithmetic operators.
  E3Timestamp& operator+=(const E3Timestamp& rhs);

  friend E3Timestamp operator+(E3Timestamp lhs, const E3Timestamp& rhs)
  { lhs += rhs;
    return lhs; }

  E3Timestamp& operator-=(const E3Timestamp& rhs);

  friend E3Timestamp operator-(E3Timestamp lhs, const E3Timestamp& rhs)
  { lhs -= rhs;
    return lhs; }

  /// \brief Cast to double.
  operator double() const { return (m_seconds + m_nanoseconds/1000000000.); }
 
  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3Timestamp& t)
  { return t.fillStream(os); }

 private:
  /// \brief The timestamp seconds.
  long long m_seconds;

  /// \brief The timestamp nanoseconds.
  /// Note that this field can be negative, as otherwise there's essentially
  /// no way of correctly representing negative timestamp differencese
  /// corresponding, e.g., to -0.1 s.
  long m_nanoseconds;
};

#endif
