/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3RawDataFile.h"
#include "E3Option.h"
#include "E3OptionParser.h"
#include "E3BaseEvent.h"
#include "E3Timestamp.h"
#include "E3EventList.h"
#include "E3TerminalFormatting.h"


int main (int argc, char* argv[]) {

  E3TerminalFormatting::startmsg();

  // Setup command-line options.
  std::string intent = "Read an arbitrary number of binary data (.dat) files and search for coincidences\nin time between the events therein. Note that, for debugging purposes, a +/- 1 s\noffset in the event timestamps can be added on the fly by appending a '+' or '-'\ncharacter to any given file path.";
  std::string usage = "e3findcoincidences [options] file_1.dat[+/-] ... file_n.dat[+/-]";
  E3OptionParser parser(intent, usage);
  parser.addOption<int>("time-window", 'w', 1000,
                        "The time window (in ns) for the search");
  parser.addOption<std::string>("output-file", 'o',
                                "Path to the output text file", false);
  parser.parse(argc, argv);

  // Parse options.
  std::vector<std::string> arguments = parser.getArguments();
  if (arguments.size() == 0) {
    parser.parseError("Please provide at least an input file.");
  }
  double timeWindow = 1e-9*parser.getOption<int>("time-window");
  std::string outputFilePath = parser.getOption<std::string>("output-file");

  // Prepare some variables.
  E3EventList eventList;
  std::vector<std::string>::iterator filePath;

  // Process the file list and build the common event list.
  for (filePath = arguments.begin(); filePath != arguments.end(); filePath++) {
    // For debugging purposes it is handy to be able to add an offset of +/-
    // 1 s to all the timestamps in a given run. Here we follow the convention
    // that if "+" ("-") is appended to the file path, a +1 (-1) s offset is
    // added to the corresponding events.
    int offset = 0;
    unsigned int l = filePath->size();
    if (filePath->substr(l - 1, l) == "+") {
      offset = 1;
      *filePath = filePath->substr(0, l - 1);
    } else if (filePath->substr(l - 1, l) == "-") {
      offset = -1;
      *filePath = filePath->substr(0, l - 1);
    }
    // Done with the offset, read the actual file.
    E3RawDataFile inputFile;
    if (inputFile.open(*filePath)) exit(1);
    LOG(INFO) << "Processing " << *filePath << "...";
    E3RunInfo runInfo = inputFile.parseHeader();
    while (inputFile.numBytesLeft()) {
      E3DigiEvent event = inputFile.next();
      event.unpack(runInfo);
      if (offset != 0) event.offsetTimestamp(offset);
      eventList.fill(event);
    }
  }

  // Go through the event list and search for coincidences.
  E3CoincidenceList coincidenceList = eventList.searchCoincidences(timeWindow);
  std::cout << coincidenceList << std::endl;
  if (outputFilePath.size() > 0) {
    coincidenceList.write(outputFilePath);
  }
  return 0;
}
