/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3NinoMap_H
#define E3NinoMap_H

#include<iostream>

#include "E3.h"


/*! \class E3NinoMap
  \brief Small utility class for mapping NINO to TDC boards.

  The codification exploits a 32-bit bit mask---8 groups of four bits
  whose position signals the TDC and whose values indicates the NINO
  it is connected to.

  The order of the TDC in the bitmask is the following (Bx.Ty indicates
  "Board x, TDC y"):
  32      28      24      20      16      12      8       4       0 (LSB)
  | B1.T3 | B1.T2 | B1.T1 | B1.T0 | B0.T3 | B0.T2 | B0.T1 | B0.T0 |
  The values, as mentioned above, indicates the connected NINO; 0xf indicates
  that the TDC is *not* connected.

  A typical example might be
  bit_mask = 0xf54f3210,
  that should be expanded into the corrispondence table:
  Board 0, TDC 0 <-> NINO 0
  Board 0, TDC 1 <-> NINO 1
  Board 0, TDC 2 <-> NINO 2
  Board 0, TDC 3 <-> NINO 3
  Board 1, TDC 0 <-> disconnected
  Board 1, TDC 1 <-> NINO 4
  Board 1, TDC 2 <-> NINO 5
  Board 1, TDC 3 <-> disconnected

  It is also understanded that the NINO board determine the plane ID/end in the
  telescope, according to the map:
  NINO 0 <-> Top +
  NINO 1 <-> Top -
  NINO 2 <-> Mid +
  NINO 3 <-> Mid -
  NINO 4 <-> Bot +
  NINO 5 <-> Bot -
*/


class E3NinoMap
{
 public:
  /// \brief Default constructor.
  E3NinoMap(E3::NinoMap_t payload);

  /// \brief Destructor.
  ~E3NinoMap() {}

  /// \brief Return the NINO ID cooresponding to a given board/TDC.
  E3::Nino_t nino(E3::Board_t board, E3::Tdc_t tdc) const;

  /// \brief Return the plane corresponding to a given board/TDC.
  E3::Plane_t plane(E3::Board_t board, E3::Tdc_t tdc) const;
 
  /// \brief Return the side (+/-) corresponding to a given board/TDC.
  bool side(E3::Board_t board, E3::Tdc_t tdc) const;

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3NinoMap& m)
  { return m.fillStream(os); }

 private:
  /// \brief The underlying bit mask.
  E3::NinoMap_t m_payload;
};

#endif
